@@ -861,13 +861,14 @@ static bool isOpenScriptLanguagePhp (int c)
 	return true;
 }
 
-static int findPhpStart (void)
+static int findPhpStart (int *tagStartColumn)
 {
 	int c;
 	do
 	{
 		if ((c = getcFromInputFile ()) == '<')
 		{
+			*tagStartColumn = getInputColumnNumber () - 1;
 			c = getcFromInputFile ();
 			/* <?, <?= and <?php, but not <?xml */
 			if (c == '?')
@@ -932,17 +933,20 @@ getNextChar:
 	{
 		unsigned long startSourceLineNumber = getSourceLineNumber ();
 		unsigned long startLineNumber = getInputLineNumber ();
-		int startLineOffset = getInputLineOffset ();
 
-		c = findPhpStart ();
+		int startColumnNumber = getInputColumnNumber ();
+		int endColumnNumber;
+
+		c = findPhpStart (&endColumnNumber);
 		if (c != EOF)
 			InPhp = true;
+		else
+			endColumnNumber = getInputColumnNumber ();
 
 		unsigned long endLineNumber = getInputLineNumber ();
-		int endLineOffset = getInputLineOffset ();
 
-		makePromise ("HTML", startLineNumber, startLineOffset,
-					 endLineNumber, endLineOffset, startSourceLineNumber);
+		makePromise ("HTML", startLineNumber, startColumnNumber,
+					 endLineNumber, endColumnNumber, startSourceLineNumber);
 	}
 	else
 		c = getcFromInputFile ();
@@ -1736,6 +1740,36 @@ static bool parseNamespace (tokenInfo *const token)
 	return true;
 }
 
+/* skip trait uses not to confuse typerefs
+ * 	use Name;
+ * 	use \\Some\\Name;
+ * 	use Name, Other;
+ * 	use Name { method as private; }
+ * 	use Name, Other { otherMethod as public otherName; }
+ *
+ * Note that curly braces are only allowed on the last `use`, this is a syntax
+ * error:
+ * 	use Name { method as private; }, Other;
+ * This has to be split in two `use`es or reordered.
+ */
+static bool parseClassUse (tokenInfo *const token)
+{
+	do
+	{
+		readToken (token);
+		if (token->type == TOKEN_OPEN_CURLY)
+		{
+			enterScope (token, NULL, -1);
+			return true;
+		}
+	}
+	while (token->type == TOKEN_IDENTIFIER ||
+		   token->type == TOKEN_BACKSLASH ||
+		   token->type == TOKEN_COMMA);
+
+	return (token->type == TOKEN_SEMICOLON);
+}
+
 static void enterScope (tokenInfo *const parentToken,
 						const vString *const extraScope,
 						const int parentKind)
@@ -1798,6 +1832,8 @@ static void enterScope (tokenInfo *const parentToken,
 						 * is also used to i.e. "import" traits into a class */
 						if (vStringLength (token->scope) == 0)
 							readNext = parseUse (token);
+						else if (parentKind == K_CLASS || parentKind == K_TRAIT)
+							readNext = parseClassUse (token);
 						break;
 
 					case KEYWORD_namespace:	readNext = parseNamespace (token);	break;
